Option Explicit On 
Option Strict On
Option Compare Binary

Imports Microsoft.VisualBasic
Imports System
Imports System.Drawing
Imports System.Runtime.InteropServices

' <remarks>
'   Basisklasse, die <c>System.Windows.Forms.Form</c> 
'   um Eigenschaften zur Grsseneinstellung erweitert.
' </remarks>
'
' Sollte MustInherit deklariert werden, was 
' aber dem Windows Forms-Designer Probleme macht.
'
Public Class ExtendedForm
    Inherits System.Windows.Forms.Form

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure POINTAPI
        Public x As Int32
        Public y As Int32
    End Structure

    ' Wir subclassen nach dieser Message.
    Private Const WM_GETMINMAXINFO As Int32 = &H24

    <StructLayout(LayoutKind.Sequential)> _
    Private Structure MINMAXINFO
        Public ptReserved As POINTAPI
        Public ptMaxSize As POINTAPI
        Public ptMaxPosition As POINTAPI
        Public ptMinTrackSize As POINTAPI
        Public ptMaxTrackSize As POINTAPI
    End Structure

    ' Deklarationen fr die verwendeten API-Funktionen.
    Private Declare Sub CopyMemory Lib "kernel32.dll" _
      Alias "RtlMoveMemory" ( _
      ByVal lpDest As IntPtr, _
      ByRef lpSource As MINMAXINFO, _
      ByVal cBytes As Int32)

    Private Declare Sub CopyMemory Lib _
      "kernel32.dll" Alias "RtlMoveMemory" ( _
      ByRef lpDest As MINMAXINFO, _
      ByVal lpSource As IntPtr, _
      ByVal cBytes As Int32)

    ' Private Membervariablen zum Speichern #
    ' der Eigenschaftswerte.
    Private m_ptMaximizedLocation As Point
    Private m_sizeMaxTrackSize As Size
    Private m_sizeMaximumSize As Size
    Private m_sizeMinimumSize As Size

#Region " Vom Windows Form Designer generierter Code "
    Public Sub New()
        MyBase.New()
        InitializeComponent()
    End Sub

    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    Private components As System.ComponentModel.IContainer

    <System.Diagnostics.DebuggerStepThrough()> _
    Private Sub InitializeComponent()
        '
        'ExtendedForm
        '
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(272, 206)
        Me.Name = "ExtendedForm"
        Me.Text = "ExtendedForm"

    End Sub
#End Region

    Protected Overrides Sub WndProc(ByRef m As System.Windows.Forms.Message)
        Select Case m.Msg
            Case WM_GETMINMAXINFO

                ' Dimensionieren einer Variablen, um die Daten 
                ' fr die von Windows in lParam
                ' bergebenen Daten zu halten.
                Dim mmi As MINMAXINFO

                ' Kopieren der Struktur, die an lParam im 
                ' Speicher steht, in unseren UDT.
                Call CopyMemory(mmi, m.LParam, Len(mmi))

                With mmi
                    If Me.MaximumSize.Width <> 0 Then
                        .ptMaxSize.x = Me.MaximumSize.Width
                    End If
                    If Me.MaximumSize.Height <> 0 Then
                        .ptMaxSize.y = Me.MaximumSize.Height
                    End If
                    If Me.MaximumTrackSize.Width <> 0 Then
                        .ptMaxTrackSize.x = Me.MaximumTrackSize.Width
                    End If
                    If Me.MaximumTrackSize.Height <> 0 Then
                        .ptMaxTrackSize.y = Me.MaximumTrackSize.Height
                    End If
                    If Me.MinimumSize.Width <> 0 Then
                        .ptMinTrackSize.x = Me.MinimumSize.Width
                    End If
                    If Me.MinimumSize.Height <> 0 Then
                        .ptMinTrackSize.y = Me.MinimumSize.Height
                    End If
                    .ptMaxPosition.x = Me.MaximizedLocation.X
                    .ptMaxPosition.y = Me.MaximizedLocation.Y
                End With

                ' Zurckkopieren der angepassten Struktur an Windows.
                Call CopyMemory(m.LParam, mmi, Len(mmi))

                ' Zurckgeben von 0, das bedeutet, dass wir 
                ' die Message abgearbeitet haben.
                m.Result = IntPtr.Zero

                ' Ausstieg aus der Funktion, ohne VB seine 
                ' kleinen Hndchen an die Message
                ' zu lassen.
                Return
        End Select

        ' Meldung an VB .NET weiterreichen.
        MyBase.WndProc(m)
    End Sub

    ' <summary>
    '   Gibt die minimale Grsse des Formulars 
    '   an oder gibt sie zurck.
    ' </summary>
    ' <value></value>
    '
    ' berschatten wre wahrscheinlich nicht erforderlich.
    '
    Public Shadows Property MinimumSize() As Size
        Get
            Return m_sizeMinimumSize
        End Get
        Set(ByVal Value As Size)
            m_sizeMinimumSize = Value
        End Set
    End Property

    ' <summary>
    '   Gibt die maximale Grsse des Formulars 
    '   an oder gibt sie zurck.
    ' </summary>
    ' <value>Maximale Grsse des Formulars.</value>
    '
    ' berschatten wre wahrscheinlich nicht erforderlich.
    '
    Public Shadows Property MaximumSize() As Size
        Get
            Return m_sizeMaximumSize
        End Get
        Set(ByVal Value As Size)
            m_sizeMaximumSize = Value
        End Set
    End Property

    ' <summary>
    '   Gibt die Position des Formulars, wenn 
    '   dieses in seiner Grsse begrenzt
    '   und maximiert dargestellt wird, an 
    '   oder gibt sie zurck.
    ' </summary>
    ' <value>Position des Formulars im maximierten Zustand.</value>
    Public Property MaximizedLocation() As Point
        Get
            Return m_ptMaximizedLocation
        End Get
        Set(ByVal Value As Point)
            m_ptMaximizedLocation = Value
        End Set
    End Property

    ' <summary>
    '   Gibt die maximale Track-Grsse des 
    '   Formulars an oder gibt sie zurck.
    ' </summary>
    ' <value>Maximale Track-Grsse des Formulars.</value>
    '
    ' Diese Eigenschaft ist fr die Verwendung des 
    ' Formulars auf mehreren Bildschirmen interessant.
    ' Nheres dazu in der Dokumentation zu MINMAXINFO 
    ' in der MSDN.
    '
    Public Property MaximumTrackSize() As Size
        Get
            Return m_sizeMaxTrackSize
        End Get
        Set(ByVal Value As Size)
            m_sizeMaxTrackSize = Value
        End Set
    End Property
End Class